@@ -12,7 +12,7 @@ use YAML;
 use URI::Encode qw/uri_encode/;
 use Encode;
 
-our $VERSION = 0.012;
+our $VERSION = 0.013;
 
 =head1 NAME
 
@@ -394,6 +394,7 @@ sub update_ticket {
         \@_,
         ticket_id	=> { isa    => 'Int' },
 	body		=> { isa    => 'HashRef' },
+	no_cache        => { isa    => 'Bool', optional => 1 }
     );
 
     my $encoded_body = encode_json( $params{body} );
@@ -403,6 +404,7 @@ sub update_ticket {
 	    path    => '/tickets/' . $params{ticket_id} . '.json',
 	    body    => $encoded_body,
 	);
+    $self->cache_set( 'ticket-' . $params{ticket_id}, $response ) unless( $params{no_cache} );
     return $response;
 }
 
@@ -452,7 +454,71 @@ sub get_ticket {
     return $ticket;
 }
 
-=item get_organizationt
+=item get_many_tickets
+
+Access L<Show Many Organizations|https://developer.zendesk.com/rest_api/docs/core/organizations#show-many-organizations> interface.
+
+=over 4
+
+=item ticket_ids
+
+Required.  ArrayRef of ticket ids to get
+
+=item no_cache
+
+Disable cache get/set for this operation
+
+=back
+
+Returns an array of ticket HashRefs
+
+=cut
+sub get_many_tickets {
+    my ( $self, %params ) = validated_hash(
+        \@_,
+        ticket_ids    => { isa    => 'ArrayRef' },
+        no_cache      => { isa    => 'Bool', optional => 1 }
+    );
+
+    # First see if we already have any of the ticket in our cache - less to get
+    my @tickets;
+    my @get_tickets;
+    foreach my $ticket_id ( @{ $params{ticket_ids} } ){
+        my $ticket;
+        $ticket = $self->cache_get( 'ticket-' . $ticket_id ) unless( $params{no_cache} );
+        if( $ticket ){
+            $self->log->debug( "Found ticket in cache: $ticket_id" );
+            push( @tickets, $ticket );
+        }else{
+            push( @get_tickets, $ticket_id );
+        }
+    }
+
+    # If there are any tickets remaining, get these with a bulk request
+    if( scalar( @get_tickets ) > 0 ){
+	$self->log->debug( "Tickets not in cache, requesting fresh: " . join( ',', @get_tickets ) );
+
+	#limit each request to 100 tickets per api spec
+	my @split_tickets;
+	push @split_tickets, [ splice @get_tickets, 0, 100 ] while @get_tickets;
+
+	foreach my $cur_tickets ( @split_tickets ) {
+	    my @result= $self->_paged_get_request_from_api(
+		field   => 'tickets',
+		method  => 'get',
+		path    => '/tickets/show_many.json?ids=' . join( ',', @{ $cur_tickets } ),
+		);
+	    foreach( @result ){
+		$self->log->debug( "Writing ticket to cache: $_->{id}" );
+		$self->cache_set( 'ticket-' . $_->{id}, $_ ) unless( $params{no_cache} );
+		push( @tickets, $_ );
+	    }
+	}
+    }
+    return @tickets;
+}
+
+=item get_organization
 
 Get a single organization by accessing L<Getting Organizations|https://developer.zendesk.com/rest_api/docs/core/organizations#list-organizations>
 interface with a single organization_id.  The get_many_organizations interface detailed below is more efficient for getting many organizations
@@ -479,7 +545,7 @@ sub get_organization {
         organization_id	=> { isa    => 'Int' },
         no_cache        => { isa    => 'Bool', optional => 1 }
     );
-    
+
     my $organization;
     $organization = $self->cache_get( 'organization-' . $params{organization_id} ) unless( $params{no_cache} );
     if( not $organization ){
@@ -523,10 +589,10 @@ sub get_many_organizations {
         organization_ids    => { isa    => 'ArrayRef' },
         no_cache            => { isa    => 'Bool', optional => 1 }
     );
-    
+
     # First see if we already have any of the organizations in our cache - less to get
     my @organizations;
-    my @get_organizations;
+    my @get_organization_ids;
     foreach my $org_id ( @{ $params{organization_ids} } ){
         my $organization;
         $organization = $self->cache_get( 'organization-' . $org_id ) unless( $params{no_cache} );
@@ -534,18 +600,28 @@ sub get_many_organizations {
             $self->log->debug( "Found organization in cache: $org_id" );
             push( @organizations, $organization );
         }else{
-            push( @get_organizations, $org_id );
+            push( @get_organization_ids, $org_id );
         }
     }
 
     # If there are any organizations remaining, get these with a single request
-    if( scalar( @get_organizations ) > 0 ){
-	$self->log->debug( "Organizations not in cache, requesting fresh: " . join( ',', @get_organizations ) );
+    if( scalar( @get_organization_ids ) > 0 ){
+	$self->log->debug( "Organizations not in cache, requesting fresh: " . join( ',', @get_organization_ids ) );
 	my @result= $self->_paged_get_request_from_api(
 	    field   => 'organizations',
             method  => 'get',
-	    path    => '/organizations/show_many.json?ids=' . join( ',', @get_organizations ),
-	);
+	    path    => '/organizations/show_many.json?ids=' . join( ',', @get_organization_ids ),
+	    );
+
+	#if an org is not found it is dropped from the results so we need to check for this and show an warning
+	if ( $#result != $#get_organization_ids ) {
+	    foreach my $org_id ( @get_organization_ids ){
+		my $org_found = grep { $_->{id} == $org_id } @result;
+		unless ( $org_found ) {
+		    $self->log->warn( "The following organization id was not found in Zendesk: $org_id");
+		}
+	    }
+	}
         foreach( @result ){
             $self->log->debug( "Writing organization to cache: $_->{id}" );
             $self->cache_set( 'organization-' . $_->{id}, $_ ) unless( $params{no_cache} );
@@ -631,29 +707,108 @@ sub list_organization_users {
         \@_,
         organization_id	=> { isa    => 'Int' },
         no_cache        => { isa    => 'Bool', optional => 1 }
-    );
+	);
 
-    my $users_arrayref;
-    $users_arrayref = $self->cache_get( 'organization-users-' . $params{organization_id} ) unless( $params{no_cache} );
+    # for caching we store an array of user ids for each organization and attempt to get these from the cache
+    my $user_ids_arrayref = $self->cache_get( 'organization-users-ids-' . $params{organization_id} ) unless( $params{no_cache} );
     my @users;
-    if( $users_arrayref ){
-        @users = @{ $users_arrayref };
-        $self->log->debug( sprintf "Users from cache for organization: %u", scalar( @users ), $params{organization_id} );
+    if( $user_ids_arrayref ){
+        $self->log->debug( sprintf "Users from cache for organization_id: %u", scalar(  @{ $user_ids_arrayref } ), $params{organization_id} );
+	#get the data for each ticket in the ticket array
+	my @ticket_data = $self->get_many_users (
+	    user_ids => $user_ids_arrayref,
+	    no_cache => $params{no_cache},
+	    );
+	push (@users, @ticket_data);
+
     }else{
-        $self->log->debug( "Requesting users fresh for organization: $params{organization_id}" );
+        $self->log->debug( "Requesting users fresh for user: $params{organization_id}" );
         @users = $self->_paged_get_request_from_api(
             field   => 'users',
             method  => 'get',
             path    => '/organizations/' . $params{organization_id} . '/users.json',
         );
 
-	$self->cache_set( 'organization-users-' . $params{organization_id}, \@users ) unless( $params{no_cache} );
+        $user_ids_arrayref = [ grep{ $_->{id} } @users ];
+
+	$self->cache_set( 'organization-users-ids-' . $params{organization_id}, $user_ids_arrayref ) unless( $params{no_cache} );
+        foreach( @users ){
+	    $self->log->debug( "Writing ticket to cache: $_->{id}" );
+	    $self->cache_set( 'user-' . $_->{id}, $_ ) unless( $params{no_cache} );
+        }
     }
     $self->log->debug( sprintf "Got %u users for organization: %u", scalar( @users ), $params{organization_id} );
 
     return @users;
 }
 
+
+=item get_many_users
+
+Access L<Show Many Users|https://developer.zendesk.com/rest_api/docs/core/users#show-many-users> interface.
+
+=over 4
+
+=item user_ids
+
+Required.  ArrayRef of user ids to get
+
+=item no_cache
+
+Disable cache get/set for this operation
+
+=back
+
+Returns an array of user HashRefs
+
+=cut
+
+sub get_many_users {
+    my ( $self, %params ) = validated_hash(
+        \@_,
+        user_ids    => { isa    => 'ArrayRef' },
+        no_cache      => { isa    => 'Bool', optional => 1 }
+    );
+
+    # First see if we already have any of the user in our cache - less to get
+    my @users;
+    my @get_users;
+    foreach my $user_id ( @{ $params{user_ids} } ){
+        my $user;
+        $user = $self->cache_get( 'user-' . $user_id ) unless( $params{no_cache} );
+        if( $user ){
+            $self->log->debug( "Found user in cache: $user_id" );
+            push( @users, $user );
+        }else{
+            push( @get_users, $user_id );
+        }
+    }
+
+    # If there are any users remaining, get these with a bulk request
+    if( scalar( @get_users ) > 0 ){
+	$self->log->debug( "Users not in cache, requesting fresh: " . join( ',', @get_users ) );
+
+	#limit each request to 100 users per api spec
+	my @split_users;
+	push @split_users, [ splice @get_users, 0, 100 ] while @get_users;
+
+	foreach my $cur_users (@split_users) {
+	    my @result= $self->_paged_get_request_from_api(
+		field   => 'users',
+		method  => 'get',
+		path    => '/users/show_many.json?ids=' . join( ',', @{ $cur_users } ),
+		);
+	    foreach( @result ){
+		$self->log->debug( "Writing user to cache: $_->{id}" );
+		$self->cache_set( 'user-' . $_->{id}, $_ ) unless( $params{no_cache} );
+		push( @users, $_ );
+	    }
+	}
+    }
+    return @users;
+}
+
+
 =item update_user
 
 Use the L<Update User|https://developer.zendesk.com/rest_api/docs/core/users#update-user> interface.
@@ -685,8 +840,7 @@ sub update_user {
     );
 
     my $body = {
-        "user" =>
-            $params{details}
+        "user" => $params{details}
     };
 
     my $encoded_body = encode_json( $body );
@@ -727,31 +881,69 @@ Returns array of tickets
 sub list_user_assigned_tickets {
     my ( $self, %params ) = validated_hash(
         \@_,
-        user_id	=> { isa    => 'Int' },
-        no_cache        => { isa    => 'Bool', optional => 1 }
-    );
+        user_id	    => { isa    => 'Int' },
+        no_cache    => { isa    => 'Bool', optional => 1 }
+	);
 
-    my $tickets_arrayref;
-    $tickets_arrayref = $self->cache_get( 'user-assigned-tickets-' . $params{user_id} ) unless( $params{no_cache} );
+    #for caching we store an array of ticket ids under the user, then look at the ticket cache
+    my $ticket_ids_arrayref = $self->cache_get( 'user-assigned-tickets-ids-' . $params{user_id} ) unless( $params{no_cache} );
     my @tickets;
-    if( $tickets_arrayref ){
-        @tickets = @{ $tickets_arrayref };
-        $self->log->debug( sprintf "Tickets from cache for user: %u", scalar( @tickets ), $params{user_id} );
+    if( $ticket_ids_arrayref ){
+        $self->log->debug( sprintf "Tickets from cache for user: %u", scalar( @{ $ticket_ids_arrayref } ), $params{user_id} );
+	#get the data for each ticket in the ticket array
+	@tickets = $self->get_many_tickets (
+	    ticket_ids => $ticket_ids_arrayref,
+	    no_cache => $params{no_cache},
+	    );
     }else{
         $self->log->debug( "Requesting tickets fresh for user: $params{user_id}" );
         @tickets = $self->_paged_get_request_from_api(
             field   => 'tickets',
             method  => 'get',
             path    => '/users/' . $params{user_id} . '/tickets/assigned.json',
-        );
+	    );
+        $ticket_ids_arrayref = [ grep{ $_->{id} } @tickets ];
 
-	$self->cache_set( 'user-assigned-tickets-' . $params{user_id}, \@tickets ) unless( $params{no_cache} );
+	$self->cache_set( 'user-assigned-tickets-ids-' . $params{user_id}, $ticket_ids_arrayref ) unless( $params{no_cache} );
+
+        foreach( @tickets ){
+	    $self->log->debug( "Writing ticket to cache: $_->{id}" );
+	    $self->cache_set( 'ticket-' . $_->{id}, $_ ) unless( $params{no_cache} );
+        }
     }
     $self->log->debug( sprintf "Got %u assigned tickets for user: %u", scalar( @tickets ), $params{user_id} );
 
     return @tickets;
 }
 
+
+=item clear_cache_object_id
+
+Clears an object from the cache.
+
+=over 4
+
+=item user_id
+
+Required.  Object id to clear from the cache.
+
+=back
+
+Returns whether cache_del was successful or not
+
+=cut
+sub clear_cache_object_id {
+    my ( $self, %params ) = validated_hash(
+        \@_,
+        object_id	=> { isa    => 'Str' }
+	);
+
+    $self->log->debug( "Clearing cache id: $params{object_id}" );
+    my $foo = $self->cache_del( $params{object_id} );
+
+    return $foo;
+}
+
 sub _paged_get_request_from_api {
     my ( $self, %params ) = validated_hash(
         \@_,
